package org.hardknockjava.lesson15.example01;

/********************************************************
 * This project illustrates the use of reflection to
 * create new object instances; inspect field attributes;
 * and retrieve and set field values.
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.hardknockjava.lesson15.record.BaseComparator;
import org.hardknockjava.lesson15.record.BaseRecord;
import org.hardknockjava.lesson15.record.Directory;
import org.hardknockjava.lesson15.record.Payroll;

public class Driver {
    private static Comparator<BaseRecord> baseComparator = new BaseComparator();

    public static void main(String[] args) throws Exception {
        processFile("data/directory.txt", Directory.class);

        processFile("data/payroll.txt", Payroll.class);
    }

    // processFile reads a line from the specified file,
    // creates an instance of the specified class, and
    // populates the fields of that instance.
    // Note the call to newInstance() that creates an
    // instance of the desired class using the default
    // constructor.
    private static void processFile(String filename, Class<? extends BaseRecord> clazz) throws Exception {
        // Read the file into memory.
        File inputFile = new File(filename);
        Reader r = new FileReader(inputFile);
        BufferedReader br = new BufferedReader(r);
        String line = null;

        List<BaseRecord> recordList = new ArrayList<BaseRecord>();

        try {
            while ((line = br.readLine()) != null) {
                BaseRecord record = clazz.getDeclaredConstructor().newInstance();
                record.parse(line);
                recordList.add(record);
            }
        } finally {
            br.close();
        }

        // Sort the instances using BaseComparator.
        recordList.sort(baseComparator);
        for (BaseRecord record : recordList) {
            System.out.println(record.toString());
        }

        System.out.println();
    }
}
