package org.hardknockjava.lesson15.record;

/***********************************************************
 * BaseRecord is the superclass for Directory and Payroll.
 * When a BaseRecord subclass is created, we create a List
 * of its declared fields (except static ones) and save\
 * the List in a Map.
 */

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class BaseRecord {
    // We have a field in this superclass just to illustrate
    // how we follow the class hierarchy in order to find
    // all the declared fields. Since we access the field
    // through reflection only, there is no getter or setter.
    @SuppressWarnings("unused")
    private Integer employeeNumber;

    protected static Map<Class<?>, List<Field>> fieldMap = new HashMap<Class<?>, List<Field>>();

    protected BaseRecord() {
        // If the list of fields hasn't been constructed yet,
        // do so.
        List<Field> fieldList = fieldMap.get(this.getClass()); 
        if (fieldList == null) {
            fieldList = new ArrayList<Field>();
            fieldMap.put(this.getClass(), fieldList);

            @SuppressWarnings("unchecked")
            Class<? super BaseRecord> clazz = (Class<? super BaseRecord>) this.getClass();
            while (clazz != null) {
                // TODO
                // Populate fieldList with the fields in the class
                // being implemented in order of declaration, preceded
                // by the fields in its superclass(es).
                // Remember to skip static fields!
                
                // Get the superclass and repeat.
                clazz = clazz.getSuperclass();
            }
        }
    }

    // Parse an input record into the fields of an instance of
    // the receiving class.
    public void parse(String input) throws Exception {
        String[] values = input.split(",");

        List<Field> fieldList = fieldMap.get(this.getClass());

        // Each Field in fieldList corresponds to the value at
        // the same position in values.
        for (int index = 0; index < fieldList.size() && index < values.length; index++) {
            Field field = fieldList.get(index);
            String value = values[index];
            
            // TODO
            // Copy "value" to the field described by "field".
            // To populate Integer and Float values, you need
            // to parse "value" into the appropriate type.
            
        }
    }

    // toString concatentates field names with values.
    public String toString() {
        StringBuffer buf = new StringBuffer();

        List<Field> fieldList = fieldMap.get(this.getClass());
        for (Field field : fieldList) {
            boolean accessible = field.canAccess(this);
            field.setAccessible(true);
            try {
                buf.append(field.getName() + "=" + field.get(this) + "  ");
            } catch (IllegalArgumentException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
            field.setAccessible(accessible);
        }

        return buf.toString().trim();
    }
}
